
/*
  Remote VPLUS invocation facility
  Version 1.0 / 04.apr.02 - 20.apr.02
  Copyright (C) 2002  Lars Appel

  This package is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This package is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this package; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.io.*;
import java.net.*;


public class Disp extends javax.swing.JFrame implements Runnable {

  static int disp_port;
  static String serv_addr;
  static int serv_port;

  Socket s;
  Thread t;

  /** Initializes the Form */
  public Disp(Socket sock) {
    s = sock;
    initComponents ();
    pack ();
    t = new Thread(this);
    t.start();
  }

  /** This thread will relay and display vplus calls */

  public void run() {

    try {

      System.out.println( "Vplus Trace Display thread started" );

      InputStream client_is = s.getInputStream();
      OutputStream client_os = s.getOutputStream();

      Socket so = new Socket(serv_addr, serv_port);

      InputStream server_is = so.getInputStream();
      OutputStream server_os = so.getOutputStream();

      byte[] buf = new byte[Defs.BUF_SZ];

      textArea.append("-client started-\n");

      for (;;) {
        buf_recv(buf, client_is);
        textArea.append("req " + buf_peek(buf)); textArea.append("\n");
        textArea.setCaretPosition(textArea.getDocument().getLength());
        buf_send(buf, server_os);
        if (buf_int(buf, 8) == Defs.R_FINISH) break;
        buf_recv(buf, server_is);
        textArea.append("rsp " + buf_peek(buf)); textArea.append("\n");
        textArea.setCaretPosition(textArea.getDocument().getLength());
        buf_send(buf, client_os);
      }

      textArea.append("-client stopped-\n");

      server_os.close();
      server_is.close();
      so.close();

      client_os.close();
      client_is.close();
      s.close();

    } catch (IOException e) {
      textArea.append("-client stopped-\n");
      textArea.append(e.getMessage());
    }

  }

  private int buf_int(byte[] buf, int offs) {
    int x = 0;
    x |= ( buf[offs+0] & 0xff ) << 24;
    x |= ( buf[offs+1] & 0xff ) << 16;
    x |= ( buf[offs+2] & 0xff ) <<  8;
    x |= ( buf[offs+3] & 0xff )      ;
    return x;
  }

  private void buf_send(byte[] buf, OutputStream st) throws IOException {
    // System.out.println("send " + buf_int(buf, 0));
    try {
      st.write(buf, 0, buf_int(buf, 0));
    }
    catch (IOException e) {
      textArea.append("-problem in buf_send method-\n");
      textArea.append(e.getMessage());
      throw e;
    }
  }

  private void buf_recv(byte[] buf, InputStream st) throws IOException {
    try {
      int have = 0;
      while (have < 4) {
        int part = st.read(buf, have, 4 - have);
        if (part == -1) throw new IOException("lost connection?\n");
        have += part;
      }
      // System.out.println("recv " + buf_int(buf, 0));
      int want = buf_int(buf, 0);
      while (have < want) {
        int part = st.read(buf, have, want - have);
        if (part == -1) throw new IOException("lost connection??\n");
        have += part;
      }
    }
    catch (IOException e) {
      textArea.append("-problem in buf_recv method-\n");
      textArea.append(e.getMessage());
      throw e;
    }
  }

  private String buf_peek(byte[] buf) {
    if (buf_int(buf, 0) < 12) return "BUF_TOO_SMALL";
    if (buf_int(buf, 4) != 4) return "REQ_NOT_VALID";
    return Defs.toString(buf_int(buf, 8));
  }

  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the FormEditor.
   */
  private void initComponents () {//GEN-BEGIN:initComponents
    setTitle ("Vplus Trace Display");
    addWindowListener (new java.awt.event.WindowAdapter () {
        public void windowClosing (java.awt.event.WindowEvent evt) {
          exitForm (evt);
        }
      }
    );
    getContentPane ().setLayout (new java.awt.BorderLayout ());

    scrollPane = new javax.swing.JScrollPane ();
    scrollPane.setPreferredSize (new java.awt.Dimension(400, 200));

      textArea = new javax.swing.JTextArea ();
      textArea.setEditable (false);

    scrollPane.setViewportView (textArea);
    getContentPane ().add (scrollPane, "Center");

    bottomPanel = new javax.swing.JPanel ();
    bottomPanel.setLayout (new java.awt.FlowLayout ());

      clearButton = new javax.swing.JButton ();
      clearButton.setText ("clear");
      clearButton.addActionListener (new java.awt.event.ActionListener () {
          public void actionPerformed (java.awt.event.ActionEvent evt) {
            clrButtonActionPerformed (evt);
          }
        }
      );
      bottomPanel.add (clearButton);

    getContentPane ().add (bottomPanel, "South");

  }//GEN-END:initComponents

  private void clrButtonActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_clrButtonActionPerformed
    // Add your handling code here:
    textArea.setText(null);
  }//GEN-LAST:event_clrButtonActionPerformed

  /** Exit the Application */
  private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
    System.exit (0);
  }//GEN-LAST:event_exitForm


// Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JScrollPane scrollPane;
  private javax.swing.JPanel bottomPanel;
  private javax.swing.JTextArea textArea;
  private javax.swing.JButton clearButton;
// End of variables declaration//GEN-END:variables


  public static void main(java.lang.String[] args) {

    if (args.length > 0)
      disp_port = Integer.parseInt(args[0]);
    else
      disp_port = 12345;

    if (args.length > 1)
      serv_addr = args[1];
    else
      serv_addr = "127.0.0.1";

    if (args.length > 2)
      serv_port = Integer.parseInt(args[2]);
    else
      serv_port = 23456;

    // new Disp ().show ();

    try {

      ServerSocket serv = new ServerSocket(disp_port);

      System.out.println( "Vplus Trace Display listener ready" );

      for (;;) {
        Socket sock = serv.accept();
        new Disp(sock) .show();
      }

      // serv.close();

    } catch (IOException e) {
      e.printStackTrace();
    }

  }

}

